import datetime
import httplib
import json

# from baluhn import verify

from . import RedPayRequest, RedPayResponse, Packet, aesEncryptor


class CreditCardProcessor:
    def __init__(self, config):
        self.config = config

    def Charge(self, amount=0, account=None, currency="USD", expDate=None, cvv=None, signatureData=None,
               cardHolderName=None, method=None, authCode=None, retryCount=None, track1Data=None, track2Data=None,
               avsAddress1=None, avsAddress2=None, avsCity=None, avsZip=None, cardHolderEmail=None,
               cardHolderPhone=None, employeeRefNum=None, customerRefNum=None, orderRefNum=None, terminalRefNum=None):

        # Creates a RedPayResponse object
        req = RedPayRequest.RedPayRequest(
            mAction="A",
            mAmount=amount,
            mAccount=account,
            mCurrency=currency,
            mExpDate=expDate,
            mCvv=cvv,
            mSignatureData=signatureData,
            mCardHolderName=cardHolderName,
            mMethod=method,
            mAuthCode=authCode,
            mRetryCount=retryCount,
            mTrack1Data=track1Data,
            mTrack2Data=track2Data,
            mAvsAddress1=avsAddress1,
            mAvsAddress2=avsAddress2,
            mAvsCity=avsCity,
            mAvsZip=avsZip,
            mCardHolderEmail=cardHolderEmail,
            mCardHolderPhone=cardHolderPhone,
            mEmployeeRefNum=employeeRefNum,
            mCustomerRefNum=customerRefNum,
            mOrderRefNum=orderRefNum,
            mTerminalRefNum=terminalRefNum
        )

        # Raises an exception to save a server call if an invalid request is built
        self.__validateCharge(req)

        # Returns RedPayResponse object
        return self.sendRequest(req)

    def Void(self, transactionId=None):

        # Returns a RedPayResponse object
        req = RedPayRequest.RedPayRequest(
            mAction="V",
            mTransactionId=transactionId
        )

        # Raises an exception to save a server call if an invalid request is built
        self.__validateVoid(req)

        # Returns RedPayResponse object
        return self.sendRequest(req)

    def Refund(self, transactionId=None, amount=0):

        # Returns a RedPayResponse object
        req = RedPayRequest.RedPayRequest(
            mAction="R",
            mTransactionId=transactionId,
            mAmount=amount
        )

        # Raises an exception to save a server call if an invalid request is built
        self.__validateRefund(req)

        # Returns RedPayResponse object
        return self.sendRequest(req)

    def sendRequest(self, req):

        # Encrypts RedPayRequest
        jsonRequest = json.dumps(req.__dict__)
        enc = aesEncryptor.encrypt(self.config.base64Key, jsonRequest)
        randIvString = enc[0]
        aesData = enc[1]

        # Creates Packet object and POSTs it
        packet = Packet(
            app=self.config.appName,
            aesData=aesData,
            iv=randIvString,
            utcTime=str(datetime.datetime.utcnow())
        )

        # Opens a secure connection with demo site
        connection = httplib.HTTPSConnection(self.config.url)

        headers = {
            "content-type": "application/json",
        }

        # POSTS the packet and gets a response
        jsonPacket = json.dumps(packet.__dict__)
        connection.request("POST", "/test", jsonPacket, headers)
        response = connection.getresponse().read()
        responsePacket = json.loads(response.decode("utf-8"))

        # Decodes the encrypted response
        aesData = str(responsePacket['aesData'])
        iv = str(responsePacket['iv'])
        redPayResponseText = aesEncryptor.decrypt(self.config.base64Key, iv, aesData)

        # Maps json to response object and returns it
        j = json.loads(redPayResponseText)
        return RedPayResponse.RedPayResponse(**j)

    def __validateCharge(self, req):
        # Validation of required fields
        if not type(req.amount) is int or req.amount <= 0:
            raise RuntimeError("Missing/Invalid amount. Please provide a positive integer")
        if not type(req.account) is str or not req.account.isdigit() or int(req.account) <= 0:
            raise RuntimeError("Missing/Invalid account number. Please provide in a valid digit string")
        if not type(req.cardHolderName) is str:
            raise RuntimeError("Missing/Invalid cardholder name. Cardholder's name must be a string.")
        if not type(req.expmmyyyy) is str or not req.expmmyyyy.isdigit() or not len(req.expmmyyyy) == 6:
            raise RuntimeError("Missing/Invalid expiration date. Please provide a digit string in the form 'mmyyyy")
        if not type(req.cvv) is str or not req.cvv.isdigit():
            raise RuntimeError("Missing/Invalid cvv. Please provide a digit string")
        if not type(req.avsZip) is str or not req.avsZip.isdigit():
            raise RuntimeError("Missing/Invalid zip code. Please provide a digit string")
        # if not verify(req.account):
        #     raise RuntimeError("Invalid credit card number. Failed Baluhn verification")

    def __validateVoid(self, req):
        if not type(req.transactionId) is str:
            raise RuntimeError("Missing/Invalid transactionId is not a string")

    def __validateRefund(self, req):
        if not type(req.transactionId) is str:
            raise RuntimeError("Missing/Invalid transactionId is not a string")
        if not type(req.amount) is int or req.amount <= 0:
            raise RuntimeError("Missing/Invalid amount. Please provide a positive integer")
